@@ -329,9 +329,9 @@ function setup_mvfst() {
   MVFST_BUILD_DIR=$DEPS_DIR/mvfst/build/
   if [ ! -d "$MVFST_DIR" ] ; then
     echo -e "${COLOR_GREEN}[ INFO ] Cloning mvfst repo ${COLOR_OFF}"
-    git clone https://github.com/facebookincubator/mvfst "$MVFST_DIR"
+    git clone https://github.com/facebook/mvfst "$MVFST_DIR"
   fi
-  synch_dependency_to_commit "$MVFST_DIR" "$BASE_DIR"/../build/deps/github_hashes/facebookincubator/mvfst-rev.txt
+  synch_dependency_to_commit "$MVFST_DIR" "$BASE_DIR"/../build/deps/github_hashes/facebook/mvfst-rev.txt
   echo -e "${COLOR_GREEN}Building Mvfst ${COLOR_OFF}"
   mkdir -p "$MVFST_BUILD_DIR"
   cd "$MVFST_BUILD_DIR" || exit
@@ -360,20 +360,16 @@ function setup_mvfst() {
 
 # Parse args
 JOBS=8
-WITH_QUIC=false
 INSTALL_DEPENDENCIES=true
 FETCH_DEPENDENCIES=true
 PREFIX=""
 COMPILER_FLAGS=""
-USAGE="./build.sh [-j num_jobs] [-q|--with-quic] [-m|--no-jemalloc] [--no-install-dependencies] [-p|--prefix] [-x|--compiler-flags] [--no-fetch-dependencies]"
+USAGE="./build.sh [-j num_jobs] [-m|--no-jemalloc] [--no-install-dependencies] [-p|--prefix] [-x|--compiler-flags] [--no-fetch-dependencies]"
 while [ "$1" != "" ]; do
   case $1 in
     -j | --jobs ) shift
                   JOBS=$1
                   ;;
-    -q | --with-quic )
-                  WITH_QUIC=true
-                  ;;
     -m | --no-jemalloc )
                   NO_JEMALLOC=true
                   ;;
@@ -433,11 +429,7 @@ setup_zstd
 setup_folly
 setup_fizz
 setup_wangle
-MAYBE_BUILD_QUIC=""
-if [ "$WITH_QUIC" == true ] ; then
-  setup_mvfst
-  MAYBE_BUILD_QUIC="-DBUILD_QUIC=On"
-fi
+setup_mvfst
 
 MAYBE_BUILD_FUZZERS=""
 MAYBE_USE_STATIC_DEPS=""
@@ -464,7 +456,6 @@ cmake                                     \
   -DCMAKE_BUILD_TYPE=RelWithDebInfo       \
   -DCMAKE_PREFIX_PATH="$DEPS_DIR"         \
   -DCMAKE_INSTALL_PREFIX="$PREFIX"        \
-  "$MAYBE_BUILD_QUIC"                     \
   "$MAYBE_BUILD_TESTS"                    \
   "$MAYBE_BUILD_FUZZERS"                  \
   "$MAYBE_BUILD_SHARED_LIBS"              \
