@@ -37,6 +37,7 @@ namespace proxygen {
 #endif
 
 #else
+#include <stdbool.h>
 #define nullptr NULL
 
 #endif /* __cplusplus */
@@ -226,10 +227,19 @@ static const int8_t unhex[256] =
   ,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1
   };
 
-#if HTTP_PARSER_STRICT
-# define T(v) 0
+#if HTTP_PARSER_STRICT_URL
+/* The value here is "0b10000000" so when shifted by
+ * F_PARSE_URL_OPTIONS_URL_STRICT/F_HTTP_PARSER_OPTIONS_URL_STRICT it remains
+ * > 0 for non-strict (0) and becomes 0 for strict (1 << 0 = 1)
+ */
+static_assert(F_PARSE_URL_OPTIONS_URL_STRICT == 1, "must be 1!");
+static_assert(F_HTTP_PARSER_OPTIONS_URL_STRICT == 1, "must be 1!");
+# define T(v) 0x80
+# define IS_PARSER_STRICT(options)                                             \
+  ((options) & F_HTTP_PARSER_OPTIONS_URL_STRICT)
 #else
 # define T(v) v
+# define IS_PARSER_STRICT(options) (0)
 #endif
 
 static const uint8_t normal_url_char[256] = {
@@ -408,16 +418,23 @@ enum http_host_state
   (c) == ';' || (c) == ':' || (c) == '&' || (c) == '=' || (c) == '+' || \
   (c) == '$' || (c) == ',')
 
-#if HTTP_PARSER_STRICT
-#define IS_URL_CHAR(c)      (normal_url_char[(unsigned char) (c)])
+#if HTTP_PARSER_STRICT_URL
+#define IS_URL_CHAR(c, strict)                                                 \
+  (((normal_url_char[(unsigned char) (c)] << (strict)) != 0) ||                \
+   (((c) & 0x80) && !(strict)))
+#else
+#define IS_URL_CHAR(c, strict)                                                 \
+  (normal_url_char[(unsigned char) (c)] || ((c) & 0x80))
+#endif
+
+#if HTTP_PARSER_STRICT_HOSTNAME
 #define IS_HOST_CHAR(c)     (IS_ALPHANUM(c) || (c) == '.' || (c) == '-')
 #else
-#define IS_URL_CHAR(c)                                                         \
-  (normal_url_char[(unsigned char) (c)] || ((c) & 0x80))
 #define IS_HOST_CHAR(c)                                                        \
   (IS_ALPHANUM(c) || (c) == '.' || (c) == '-' || (c) == '_')
 #endif
 
+
 /**
  * Verify that a char is a valid visible (printable) US-ASCII
  * character or %x80-FF
@@ -452,13 +469,13 @@ static struct {
  * URL and non-URL states by looking for these.
  */
 static enum state
-parse_url_char(enum state s, const char ch)
+parse_url_char(enum state s, const char ch, int strict_flag)
 {
   if (ch == ' ' || ch == '\r' || ch == '\n') {
     return s_dead;
   }
 
-#if HTTP_PARSER_STRICT
+#if HTTP_PARSER_STRICT_URL
   if (ch == '\t' || ch == '\f') {
     return s_dead;
   }
@@ -510,7 +527,11 @@ parse_url_char(enum state s, const char ch)
         return s_dead;
       }
 
+#if __cplusplus
+      [[fallthrough]];
+#else /* __cplusplus */
     /* FALLTHROUGH */
+#endif /* __cplusplus */
     case s_req_server_start:
     case s_req_server:
       if (ch == '/') {
@@ -532,7 +553,7 @@ parse_url_char(enum state s, const char ch)
       break;
 
     case s_req_path:
-      if (IS_URL_CHAR(ch)) {
+      if (IS_URL_CHAR(ch, strict_flag)) {
         return s;
       }
 
@@ -548,7 +569,7 @@ parse_url_char(enum state s, const char ch)
 
     case s_req_query_string_start:
     case s_req_query_string:
-      if (IS_URL_CHAR(ch)) {
+      if (IS_URL_CHAR(ch, strict_flag)) {
         return s_req_query_string;
       }
 
@@ -564,7 +585,7 @@ parse_url_char(enum state s, const char ch)
       break;
 
     case s_req_fragment_start:
-      if (IS_URL_CHAR(ch)) {
+      if (IS_URL_CHAR(ch, strict_flag)) {
         return s_req_fragment;
       }
 
@@ -579,7 +600,7 @@ parse_url_char(enum state s, const char ch)
       break;
 
     case s_req_fragment:
-      if (IS_URL_CHAR(ch)) {
+      if (IS_URL_CHAR(ch, strict_flag)) {
         return s;
       }
 
@@ -603,6 +624,15 @@ size_t http_parser_execute (http_parser *parser,
                             const http_parser_settings *settings,
                             const char *data,
                             size_t len)
+{
+  return http_parser_execute_options(parser, settings, 0, data, len);
+}
+
+size_t http_parser_execute_options (http_parser *parser,
+                                    const http_parser_settings *settings,
+                                    uint8_t options,
+                                    const char *data,
+                                    size_t len)
 {
   char c, ch;
   int8_t unhex_val;
@@ -966,7 +996,7 @@ size_t http_parser_execute (http_parser *parser,
         if (ch == ' ' && matcher[parser->index] == '\0') {
           state = s_req_spaces_before_url;
         } else if (ch == matcher[parser->index]) {
-          ; /* nada */
+          /* nada */
         } else if (parser->method == HTTP_CONNECT) {
           if (parser->index == 1 && ch == 'H') {
             parser->method = HTTP_CHECKOUT;
@@ -1143,7 +1173,7 @@ size_t http_parser_execute (http_parser *parser,
 
       case s_req_path:
       {
-        if (IS_URL_CHAR(ch)) break;
+        if (IS_URL_CHAR(ch, IS_PARSER_STRICT(options))) break;
 
         switch (ch) {
           case ' ':
@@ -1178,7 +1208,7 @@ size_t http_parser_execute (http_parser *parser,
 
       case s_req_query_string_start:
       {
-        if (IS_URL_CHAR(ch)) {
+        if (IS_URL_CHAR(ch, IS_PARSER_STRICT(options))) {
           state = s_req_query_string;
           break;
         }
@@ -1215,7 +1245,7 @@ size_t http_parser_execute (http_parser *parser,
 
       case s_req_query_string:
       {
-        if (IS_URL_CHAR(ch)) break;
+        if (IS_URL_CHAR(ch, IS_PARSER_STRICT(options))) break;
 
         switch (ch) {
           case '?':
@@ -1250,7 +1280,7 @@ size_t http_parser_execute (http_parser *parser,
 
       case s_req_fragment_start:
       {
-        if (IS_URL_CHAR(ch)) {
+        if (IS_URL_CHAR(ch, IS_PARSER_STRICT(options))) {
           state = s_req_fragment;
           break;
         }
@@ -1287,7 +1317,7 @@ size_t http_parser_execute (http_parser *parser,
 
       case s_req_fragment:
       {
-        if (IS_URL_CHAR(ch)) break;
+        if (IS_URL_CHAR(ch, IS_PARSER_STRICT(options))) break;
 
         switch (ch) {
           case ' ':
@@ -2219,7 +2249,11 @@ http_parse_host_char(enum http_host_state s, const char ch) {
         return s_http_host;
       }
 
+#if __cplusplus
+      [[fallthrough]];
+#else /* __cplusplus */
     /* FALLTHROUGH */
+#endif /* __cplusplus */
     case s_http_host_v6_end:
       if (ch == ':') {
         return s_http_host_port_start;
@@ -2232,7 +2266,11 @@ http_parse_host_char(enum http_host_state s, const char ch) {
         return s_http_host_v6_end;
       }
 
+#if __cplusplus
+      [[fallthrough]];
+#else /* __cplusplus */
     /* FALLTHROUGH */
+#endif /* __cplusplus */
     case s_http_host_v6_start:
       if (IS_HEX(ch) || ch == ':' || ch == '.') {
         return s_http_host_v6;
@@ -2331,6 +2369,13 @@ http_parse_host(const char * buf, struct http_parser_url *u, int found_at) {
 int
 http_parser_parse_url(const char *buf, size_t buflen, int is_connect,
                       struct http_parser_url *u)
+{
+  return http_parser_parse_url_options(buf, buflen, is_connect, u, 0);
+}
+
+int
+http_parser_parse_url_options(const char *buf, size_t buflen, int is_connect,
+                              struct http_parser_url *u, uint8_t options)
 {
   enum state s;
   const char *p;
@@ -2342,7 +2387,7 @@ http_parser_parse_url(const char *buf, size_t buflen, int is_connect,
   uf = old_uf = UF_MAX;
 
   for (p = buf; p < buf + buflen; p++) {
-    s = parse_url_char(s, *p);
+    s = parse_url_char(s, *p, options & F_PARSE_URL_OPTIONS_URL_STRICT);
 
     /* Figure out the next field that we're operating on */
     switch (s) {
@@ -2364,7 +2409,11 @@ http_parser_parse_url(const char *buf, size_t buflen, int is_connect,
       case s_req_server_with_at:
         found_at = 1;
 
+#if __cplusplus
+        [[fallthrough]];
+#else /* __cplusplus */
       /* FALLTHROUGH */
+#endif /* __cplusplus */
       case s_req_server:
         uf = UF_HOST;
         break;
@@ -2382,7 +2431,7 @@ http_parser_parse_url(const char *buf, size_t buflen, int is_connect,
         break;
 
       default:
-        assert(!"Unexpected state");
+        assert(false && "Unexpected state");
         return 1;
     }
 
