@@ -19,6 +19,7 @@
 
 using folly::EventBaseManager;
 using folly::IOThreadPoolExecutor;
+using folly::IOThreadPoolExecutorBase;
 using folly::ThreadPoolExecutor;
 
 namespace proxygen {
@@ -97,23 +98,22 @@ void HTTPServer::bind(std::vector<IPConfig> const& addrs) {
   addresses_ = addrs;
 }
 
-class HandlerCallbacks : public ThreadPoolExecutor::Observer {
+class HandlerCallbacks : public IOThreadPoolExecutorBase::IOObserver {
  public:
   explicit HandlerCallbacks(std::shared_ptr<HTTPServerOptions> options)
       : options_(options) {
   }
 
-  void threadStarted(ThreadPoolExecutor::ThreadHandle* h) override {
-    auto evb = IOThreadPoolExecutor::getEventBase(h);
-    CHECK(evb) << "Invariant violated - started thread must have an EventBase";
-    evb->runInEventBaseThread([=]() {
+  void registerEventBase(folly::EventBase& evb) override {
+    evb.runInEventBaseThread([&evb, this]() {
       for (auto& factory : options_->handlerFactories) {
-        factory->onServerStart(evb);
+        factory->onServerStart(&evb);
       }
     });
   }
-  void threadStopped(ThreadPoolExecutor::ThreadHandle* h) override {
-    IOThreadPoolExecutor::getEventBase(h)->runInEventBaseThread([&]() {
+
+  void unregisterEventBase(folly::EventBase& evb) override {
+    evb.runInEventBaseThread([this]() {
       for (auto& factory : options_->handlerFactories) {
         factory->onServerStop();
       }
@@ -126,7 +126,7 @@ class HandlerCallbacks : public ThreadPoolExecutor::Observer {
 
 folly::Expected<folly::Unit, std::exception_ptr> HTTPServer::startTcpServer(
     std::shared_ptr<wangle::AcceptorFactory> inputAcceptorFactory,
-    std::shared_ptr<folly::IOThreadPoolExecutor> ioExecutor) {
+    std::shared_ptr<folly::IOThreadPoolExecutorBase> ioExecutor) {
   auto accExe = std::make_shared<IOThreadPoolExecutor>(1);
   if (!ioExecutor) {
     ioExecutor = std::make_shared<IOThreadPoolExecutor>(
@@ -150,6 +150,7 @@ folly::Expected<folly::Unit, std::exception_ptr> HTTPServer::startTcpServer(
       }
       bootstrap_.push_back(wangle::ServerBootstrap<wangle::DefaultPipeline>());
       bootstrap_[i].childHandler(acceptorFactory);
+      bootstrap_[i].useZeroCopy(options_->useZeroCopy);
       if (accConfig.enableTCPFastOpen) {
         // We need to do this because wangle's bootstrap has 2 acceptor configs
         // and the socketConfig gets passed to the SocketFactory. The number of
@@ -182,7 +183,7 @@ void HTTPServer::start(
     std::function<void()> onSuccess,
     std::function<void(std::exception_ptr)> onError,
     std::shared_ptr<wangle::AcceptorFactory> acceptorFactory,
-    std::shared_ptr<folly::IOThreadPoolExecutor> ioExecutor) {
+    std::shared_ptr<folly::IOThreadPoolExecutorBase> ioExecutor) {
   mainEventBase_ = EventBaseManager::get()->getEventBase();
 
   auto tcpStarted = startTcpServer(acceptorFactory, ioExecutor);
