@@ -33,6 +33,10 @@ DEFINE_string(path,
               "/",
               "(HQClient) url-path to send the request to, "
               "or a comma separated list of paths to fetch in parallel");
+DEFINE_string(connect_to_address,
+              "",
+              "(HQClient) Override IP address to connect to instead of "
+              "resolving the host field");
 DEFINE_int32(connect_timeout, 2000, "(HQClient) connect timeout in ms");
 DEFINE_string(httpversion, "1.1", "HTTP version string");
 DEFINE_string(protocol, "", "HQ protocol version e.g. h3-29 or hq-fb-05");
@@ -54,6 +58,7 @@ DEFINE_bool(sequential,
             false,
             "Whether to make requests sequentially or in parallel when "
             "multiple paths are provided");
+DEFINE_uint32(gap_ms, 0, "Time between consecutive requests in milliseconds");
 DEFINE_string(congestion, "cubic", "newreno/cubic/bbr/none");
 DEFINE_int32(conn_flow_control, 1024 * 1024 * 10, "Connection flow control");
 DEFINE_int32(stream_flow_control, 256 * 1024, "Stream flow control");
@@ -75,10 +80,6 @@ DEFINE_bool(early_data, false, "Whether to use 0-rtt");
 DEFINE_uint32(quic_batching_mode,
               static_cast<uint32_t>(quic::QuicBatchingMode::BATCHING_MODE_NONE),
               "QUIC batching mode");
-DEFINE_bool(quic_use_thread_local_batching, false, "Use thread local batching");
-DEFINE_uint32(quic_thread_local_delay_us,
-              1000,
-              "Thread local delay in microseconds");
 DEFINE_uint32(quic_batch_size,
               quic::kDefaultQuicMaxBatchSize,
               "Maximum number of packets that can be batched in Quic");
@@ -102,10 +103,6 @@ DEFINE_bool(use_inplace_write,
             false,
             "Transport use inplace packet build and socket writing");
 
-DEFINE_string(ccp_config,
-              "",
-              "Additional args to pass to ccp. Ccp disabled if empty string.");
-
 DEFINE_bool(send_knob_frame,
             false,
             "Send a Knob Frame to the peer when a QUIC connection is "
@@ -172,8 +169,13 @@ void initializeCommonSettings(HQToolParams& hqParams) {
     auto& clientParams = boost::get<HQToolClientParams>(hqParams.params);
     clientParams.host = FLAGS_host;
     clientParams.port = FLAGS_port;
-    clientParams.remoteAddress =
-        folly::SocketAddress(clientParams.host, clientParams.port, true);
+    if (FLAGS_connect_to_address.empty()) {
+      clientParams.remoteAddress =
+          folly::SocketAddress(clientParams.host, clientParams.port, true);
+    } else {
+      clientParams.remoteAddress = folly::SocketAddress(
+          FLAGS_connect_to_address, clientParams.port, false);
+    }
     if (!FLAGS_local_address.empty()) {
       clientParams.localAddress = folly::SocketAddress();
       clientParams.localAddress->setFromLocalIpPort(FLAGS_local_address);
@@ -197,21 +199,16 @@ void initializeTransportSettings(HQToolParams& hqUberParams) {
   if (!FLAGS_protocol.empty()) {
     hqParams.protocol = FLAGS_protocol;
     hqParams.supportedAlpns = {hqParams.protocol};
-  } else {
-    hqParams.supportedAlpns = {proxygen::kH3,
-                               proxygen::kHQ,
-                               proxygen::kH3FBCurrentDraft,
-                               proxygen::kH3CurrentDraft,
-                               proxygen::kHQCurrentDraft};
   }
 
-  hqParams.transportSettings.advertisedInitialConnectionWindowSize =
+  hqParams.transportSettings.advertisedInitialConnectionFlowControlWindow =
       FLAGS_conn_flow_control;
-  hqParams.transportSettings.advertisedInitialBidiLocalStreamWindowSize =
+  hqParams.transportSettings.advertisedInitialBidiLocalStreamFlowControlWindow =
       FLAGS_stream_flow_control;
-  hqParams.transportSettings.advertisedInitialBidiRemoteStreamWindowSize =
+  hqParams.transportSettings
+      .advertisedInitialBidiRemoteStreamFlowControlWindow =
       FLAGS_stream_flow_control;
-  hqParams.transportSettings.advertisedInitialUniStreamWindowSize =
+  hqParams.transportSettings.advertisedInitialUniStreamFlowControlWindow =
       FLAGS_stream_flow_control;
   hqParams.congestionControlName = FLAGS_congestion;
   hqParams.congestionControl =
@@ -224,15 +221,11 @@ void initializeTransportSettings(HQToolParams& hqUberParams) {
   hqParams.transportSettings.numGROBuffers_ = FLAGS_num_gro_buffers;
   hqParams.transportSettings.pacingEnabled = FLAGS_pacing;
   if (hqParams.transportSettings.pacingEnabled) {
-    hqParams.transportSettings.pacingTimerTickInterval =
+    hqParams.transportSettings.pacingTickInterval =
         std::chrono::microseconds(FLAGS_pacing_timer_tick_interval_us);
   }
   hqParams.transportSettings.batchingMode =
       quic::getQuicBatchingMode(FLAGS_quic_batching_mode);
-  hqParams.transportSettings.useThreadLocalBatching =
-      FLAGS_quic_use_thread_local_batching;
-  hqParams.transportSettings.threadLocalDelay =
-      std::chrono::microseconds(FLAGS_quic_thread_local_delay_us);
   hqParams.transportSettings.maxBatchSize = FLAGS_quic_batch_size;
   if (hqUberParams.mode == HQMode::CLIENT) {
     // There is no good reason to keep the socket around for a drain period for
@@ -258,9 +251,6 @@ void initializeTransportSettings(HQToolParams& hqUberParams) {
   if (hqUberParams.mode == HQMode::CLIENT) {
     boost::get<HQToolClientParams>(hqUberParams.params).connectTimeout =
         std::chrono::milliseconds(FLAGS_connect_timeout);
-  } else {
-    boost::get<HQToolServerParams>(hqUberParams.params).ccpConfig =
-        FLAGS_ccp_config;
   }
   hqParams.sendKnobFrame = FLAGS_send_knob_frame;
   if (hqParams.sendKnobFrame) {
@@ -276,10 +266,10 @@ void initializeTransportSettings(HQToolParams& hqUberParams) {
 
   if (FLAGS_use_ack_receive_timestamps) {
     hqParams.transportSettings.maybeAckReceiveTimestampsConfigSentToPeer.assign(
-        {.max_receive_timestamps_per_ack =
-             FLAGS_max_ack_receive_timestamps_to_send,
-         .receive_timestamps_exponent = kDefaultReceiveTimestampsExponent});
+        {.maxReceiveTimestampsPerAck = FLAGS_max_ack_receive_timestamps_to_send,
+         .receiveTimestampsExponent = kDefaultReceiveTimestampsExponent});
   }
+  hqParams.transportSettings.datagramConfig.enabled = true;
 } // initializeTransportSettings
 
 void initializeHttpServerSettings(HQToolServerParams& hqParams) {
@@ -317,6 +307,7 @@ void initializeHttpClientSettings(HQToolClientParams& hqParams) {
   hqParams.logResponse = FLAGS_log_response;
   hqParams.logResponseHeaders = FLAGS_log_response_headers;
   hqParams.sendRequestsSequentially = FLAGS_sequential;
+  hqParams.gapBetweenRequests = std::chrono::milliseconds(FLAGS_gap_ms);
 
   hqParams.earlyData = FLAGS_early_data;
   hqParams.migrateClient = FLAGS_migrate_client;
