@@ -13,7 +13,6 @@
 
 #include <proxygen/httpserver/samples/hq/HQParams.h>
 #include <proxygen/lib/http/session/HTTPTransaction.h>
-#include <quic/server/QuicCcpThreadLauncher.h>
 #include <quic/server/QuicServer.h>
 
 namespace proxygen {
@@ -45,10 +44,15 @@ class HQServer {
   // Sets/unsets "reject connections" flag on the QUIC server
   void rejectNewConnections(bool reject);
 
+  void setStatsFactory(
+      std::unique_ptr<quic::QuicTransportStatsCallbackFactory>&& statsFactory) {
+    statsFactory_ = std::move(statsFactory);
+  }
+
  private:
   HQServerParams params_;
   std::shared_ptr<quic::QuicServer> server_;
-  QuicCcpThreadLauncher quicCcpThreadLauncher_;
+  std::unique_ptr<quic::QuicTransportStatsCallbackFactory> statsFactory_;
 };
 
 class ScopedHQServer {
