@@ -32,6 +32,7 @@
 #include <folly/io/async/AsyncTimeout.h>
 #include <folly/io/async/EventBaseManager.h>
 #include <proxygen/httpserver/samples/hq/HQServer.h>
+#include <proxygen/httpserver/samples/hq/devious/DeviousBaton.h>
 #include <proxygen/lib/http/session/HTTPTransaction.h>
 
 namespace quic::samples {
@@ -780,6 +781,44 @@ class ServerPushHandler : public BaseSampleHandler {
   ServerPushTxnHandler pushTxnHandler_;
 };
 
+class DeviousBatonHandler : public BaseSampleHandler {
+ public:
+  explicit DeviousBatonHandler(const HandlerParams& params,
+                               folly::EventBase* evb)
+      : BaseSampleHandler(params), evb_(evb) {
+  }
+
+  void onHeadersComplete(
+      std::unique_ptr<proxygen::HTTPMessage> /* msg */) noexcept override;
+
+  void onWebTransportBidiStream(
+      proxygen::HTTPCodec::StreamID id,
+      proxygen::WebTransport::BidiStreamHandle stream) noexcept override;
+  void onWebTransportUniStream(
+      proxygen::HTTPCodec::StreamID id,
+      proxygen::WebTransport::StreamReadHandle* readHandle) noexcept override;
+
+  void onWebTransportSessionClose(
+      folly::Optional<uint32_t> error) noexcept override;
+
+  void onDatagram(std::unique_ptr<folly::IOBuf> datagram) noexcept override;
+
+  void onBody(std::unique_ptr<folly::IOBuf> /* chain */) noexcept override;
+
+  void onEOM() noexcept override;
+
+  void onError(const proxygen::HTTPException& /*error*/) noexcept override;
+
+  void detachTransaction() noexcept override {
+  }
+
+  folly::Optional<devious::DeviousBaton> devious_;
+  void readHandler(proxygen::WebTransport::StreamReadHandle* readHandle,
+                   folly::Try<proxygen::WebTransport::StreamData> streamData);
+  folly::EventBase* evb_{nullptr};
+  std::map<uint64_t, devious::DeviousBaton::BatonMessageState> streams_;
+};
+
 class StaticFileHandler : public BaseSampleHandler {
  public:
   StaticFileHandler(const HandlerParams& params, std::string staticRoot)
