@@ -15,6 +15,7 @@
 #include <folly/io/async/AsyncSSLSocket.h>
 #include <folly/io/async/AsyncServerSocket.h>
 #include <folly/io/async/EventBaseManager.h>
+#include <folly/logging/xlog.h>
 #include <folly/portability/GMock.h>
 #include <folly/portability/GTest.h>
 #include <folly/ssl/OpenSSLCertUtils.h>
@@ -34,14 +35,9 @@ using namespace folly::ssl;
 using namespace proxygen;
 using namespace CurlService;
 
-using folly::AsyncServerSocket;
-using folly::EventBaseManager;
-using folly::SocketAddress;
-using folly::SSLContext;
-
 namespace {
 
-const std::string kTestDir = getContainingDirectory(__FILE__).str();
+const std::string kTestDir = getContainingDirectory(XLOG_FILENAME).str();
 
 }
 
