@@ -14,6 +14,7 @@
 #include <quic/api/LoopDetectorCallback.h>
 #include <quic/api/QuicSocket.h>
 #include <quic/client/QuicClientTransport.h>
+#include <quic/common/events/FollyQuicEventBase.h>
 #include <quic/fizz/client/handshake/QuicPskCache.h>
 #include <quic/logging/QLogger.h>
 
@@ -49,6 +50,13 @@ class HQConnector : public HQSession::ConnectCallback {
   }
 
   void setTransportSettings(quic::TransportSettings transportSettings);
+  void setSupportedQuicVersions(
+      const std::vector<quic::QuicVersion>& versions) {
+    quicVersions_ = versions;
+  }
+  void setH3Settings(SettingsList settings) {
+    h3Settings_ = std::move(settings);
+  }
 
   void setQuicPskCache(std::shared_ptr<quic::QuicPskCache> quicPskCache);
 
@@ -85,6 +93,8 @@ class HQConnector : public HQSession::ConnectCallback {
   TimePoint connectStart_;
   HQUpstreamSession* session_{nullptr};
   quic::TransportSettings transportSettings_;
+  std::vector<quic::QuicVersion> quicVersions_;
+  folly::Optional<SettingsList> h3Settings_;
   std::shared_ptr<quic::QuicPskCache> quicPskCache_;
   bool useConnectionEndWithErrorCallback_{false};
 };
