@@ -175,7 +175,7 @@ TEST_F(HeaderTableTests, IncreaseCapacity) {
     EXPECT_EQ(table.add(accept.copy()), true);
   }
   EXPECT_EQ(table.size(), max);
-  EXPECT_EQ(table.getIndex(accept), 1);
+  EXPECT_EQ(table.getIndex(accept).first, 1);
   // head should be 0, tail should be 2
   max = 8;
   capacity = accept.bytes() * max;
@@ -183,7 +183,7 @@ TEST_F(HeaderTableTests, IncreaseCapacity) {
 
   EXPECT_LE(table.length(), table.getMaxTableLength(capacity));
   // external index didn't change
-  EXPECT_EQ(table.getIndex(accept), 1);
+  EXPECT_EQ(table.getIndex(accept).first, 1);
 }
 
 TEST_F(HeaderTableTests, VaryCapacity) {
