@@ -552,6 +552,18 @@ TEST_F(HQCodecTest, ZeroLengthSettings) {
   EXPECT_EQ(callbacks_.sessionErrors, 0);
 }
 
+TEST_F(HQCodecTest, InvalidSettings) {
+  std::deque<hq::SettingPair> settings{
+      {hq::SettingId::ENABLE_WEBTRANSPORT, 37}};
+  hq::writeSettings(queueCtrl_, settings);
+  parseControl(CodecType::CONTROL_UPSTREAM);
+  EXPECT_EQ(callbacks_.settings, 0);
+  EXPECT_EQ(callbacks_.streamErrors, 0);
+  EXPECT_EQ(callbacks_.sessionErrors, 1);
+  EXPECT_EQ(callbacks_.lastParseError->getHttp3ErrorCode(),
+            HTTP3::ErrorCode::HTTP_SETTINGS_ERROR);
+}
+
 TEST_F(HQCodecTest, ZeroLengthTrailers) {
   writeValidFrame(queue_, FrameType::HEADERS);
 
@@ -603,6 +615,33 @@ TEST_F(HQCodecTest, BasicConnect) {
   EXPECT_EQ(authority, headers.getSingleOrEmpty(proxygen::HTTP_HEADER_HOST));
 }
 
+TEST_F(HQCodecTest, TrimLwsInHeaderValue) {
+  HTTPMessage req = getGetRequest("/test");
+  req.getHeaders().add(HTTPHeaderCode::HTTP_HEADER_CONTENT_TYPE,
+                       " application/x-fb");
+  // adding header with lws will strip here
+  req.getHeaders().add(HTTPHeaderCode::HTTP_HEADER_USER_AGENT, "\n\t \r");
+  auto streamId = upstreamCodec_->createStream();
+  upstreamCodec_->generateHeader(queue_, streamId, req, true /* eom */);
+
+  parse();
+  EXPECT_EQ(callbacks_.messageBegin, 1);
+  EXPECT_EQ(callbacks_.headersComplete, 1);
+  EXPECT_EQ(callbacks_.messageComplete, 0);
+  EXPECT_EQ(callbacks_.streamErrors, 0);
+  EXPECT_EQ(callbacks_.sessionErrors, 0);
+
+  auto& parsedHeaders = callbacks_.msg->getHeaders();
+  EXPECT_TRUE(parsedHeaders.exists(HTTPHeaderCode::HTTP_HEADER_CONTENT_TYPE));
+  EXPECT_TRUE(parsedHeaders.exists(HTTPHeaderCode::HTTP_HEADER_USER_AGENT));
+  EXPECT_TRUE(
+      parsedHeaders.getSingleOrEmpty(HTTPHeaderCode::HTTP_HEADER_USER_AGENT)
+          .empty());
+  EXPECT_EQ(
+      parsedHeaders.getSingleOrEmpty(HTTPHeaderCode::HTTP_HEADER_CONTENT_TYPE),
+      "application/x-fb");
+}
+
 TEST_F(HQCodecTest, OnlyDataAfterConnect) {
   std::string authority = "myhost:1234";
   HTTPMessage request;
@@ -774,6 +813,23 @@ TEST_F(HQCodecTest, HighAscii) {
   EXPECT_EQ(callbacks_.sessionErrors, 0);
 }
 
+TEST_F(HQCodecTest, WebTransportProtocol) {
+  HTTPMessage req;
+  req.setMethod(HTTPMethod::CONNECT);
+  req.setHTTPVersion(1, 1);
+  req.setURL("/wt");
+  req.setSecure(true);
+  req.setUpgradeProtocol("webtransport");
+
+  auto id = upstreamCodec_->createStream();
+  upstreamCodec_->generateHeader(queue_, id, req, false);
+  parse();
+
+  EXPECT_FALSE(callbacks_.msg->isIngressWebsocketUpgrade());
+  EXPECT_NE(nullptr, callbacks_.msg->getUpgradeProtocol());
+  EXPECT_EQ("webtransport", *callbacks_.msg->getUpgradeProtocol());
+}
+
 struct FrameAllowedParams {
   CodecType codecType;
   FrameType frameType;
@@ -834,6 +890,9 @@ std::string frameParamsToTestName(
     case FrameType::FB_PUSH_PRIORITY_UPDATE:
       testName += "PushPriorityUpdate";
       break;
+    case FrameType::WEBTRANSPORT_BIDI:
+      testName += "WebTransportBidiStreamType";
+      break;
     default:
       testName +=
           folly::to<std::string>(static_cast<uint64_t>(info.param.frameType));
@@ -945,10 +1004,14 @@ INSTANTIATE_TEST_SUITE_P(
             CodecType::DOWNSTREAM, FrameType::FB_PRIORITY_UPDATE, false},
         (FrameAllowedParams){
             CodecType::DOWNSTREAM, FrameType::FB_PUSH_PRIORITY_UPDATE, false},
+        (FrameAllowedParams){
+            CodecType::DOWNSTREAM, FrameType::WEBTRANSPORT_BIDI, false},
         (FrameAllowedParams){
             CodecType::UPSTREAM, FrameType::FB_PRIORITY_UPDATE, false},
         (FrameAllowedParams){
             CodecType::UPSTREAM, FrameType::FB_PUSH_PRIORITY_UPDATE, false},
+        (FrameAllowedParams){
+            CodecType::UPSTREAM, FrameType::WEBTRANSPORT_BIDI, false},
         // HQ Upstream Ingress Control Codec
         (FrameAllowedParams){
             CodecType::CONTROL_UPSTREAM, FrameType::DATA, false},
@@ -978,6 +1041,8 @@ INSTANTIATE_TEST_SUITE_P(
         (FrameAllowedParams){CodecType::CONTROL_UPSTREAM,
                              FrameType::FB_PUSH_PRIORITY_UPDATE,
                              false},
+        (FrameAllowedParams){
+            CodecType::CONTROL_UPSTREAM, FrameType::WEBTRANSPORT_BIDI, false},
         // HQ Downstream Ingress Control Codec
         (FrameAllowedParams){
             CodecType::CONTROL_DOWNSTREAM, FrameType::DATA, false},
@@ -1006,7 +1071,10 @@ INSTANTIATE_TEST_SUITE_P(
             CodecType::CONTROL_DOWNSTREAM, FrameType::FB_PRIORITY_UPDATE, true},
         (FrameAllowedParams){CodecType::CONTROL_DOWNSTREAM,
                              FrameType::FB_PUSH_PRIORITY_UPDATE,
-                             true}),
+                             true},
+        (FrameAllowedParams){CodecType::CONTROL_DOWNSTREAM,
+                             FrameType::WEBTRANSPORT_BIDI,
+                             false}),
     frameParamsToTestName);
 
 class HQCodecTestFrameBeforeSettings
