@@ -44,7 +44,7 @@ class HQDownstreamSession : public HQSession {
   bool isDetachable(bool) const override;
 
   void attachThreadLocals(folly::EventBase*,
-                          folly::SSLContextPtr,
+                          std::shared_ptr<const folly::SSLContext>,
                           const WheelTimerInstance&,
                           HTTPSessionStats*,
                           FilterIteratorFn,
@@ -193,15 +193,17 @@ class HQDownstreamSession : public HQSession {
 
   bool erasePushStream(quic::StreamId streamId) override;
 
-  void onNewPushStream(quic::StreamId /* pushStreamId */,
-                       hq::PushId /* pushId */,
-                       size_t /* toConsume */) override {
+  void dispatchPushStream(quic::StreamId /* pushStreamId */,
+                          hq::PushId /* pushId */,
+                          size_t /* toConsume */) override {
     LOG(DFATAL) << "nope";
   }
 
   // This is the current method of creating new push IDs.
   hq::PushId createNewPushId();
 
+  bool pushAllowedByGoaway(hq::PushId pushId);
+
   // Value of the next pushId, used for outgoing push transactions
   hq::PushId nextAvailablePushId_{0};
 
