@@ -10,6 +10,7 @@
 #include <proxygen/lib/http/session/HQSession.h>
 
 #include <folly/io/async/HHWheelTimer.h>
+#include <quic/common/events/FollyQuicEventBase.h>
 
 namespace proxygen {
 
@@ -73,7 +74,7 @@ class HQUpstreamSession : public HQSession {
   bool isDetachable(bool checkSocket) const override;
 
   void attachThreadLocals(folly::EventBase*,
-                          folly::SSLContextPtr,
+                          std::shared_ptr<const folly::SSLContext>,
                           const WheelTimerInstance&,
                           HTTPSessionStats*,
                           FilterIteratorFn,
@@ -82,9 +83,6 @@ class HQUpstreamSession : public HQSession {
 
   void detachThreadLocals(bool) override;
 
-  void onNetworkSwitch(
-      std::unique_ptr<folly::AsyncUDPSocket>) noexcept override;
-
   /**
    * Returns true iff a new outgoing transaction can be made on this session
    */
@@ -103,9 +101,10 @@ class HQUpstreamSession : public HQSession {
     return static_cast<uint32_t>(numberOfIngressPushStreams());
   }
 
- private:
+ protected:
   ~HQUpstreamSession() override;
 
+ private:
   void connectTimeoutExpired() noexcept;
 
 #ifdef _MSC_VER
@@ -234,9 +233,9 @@ class HQUpstreamSession : public HQSession {
 
   void cleanupUnboundPushStreams(std::vector<quic::StreamId>&) override;
 
-  void onNewPushStream(quic::StreamId /* pushStreamId */,
-                       hq::PushId /* pushId */,
-                       size_t /* toConsume */) override;
+  void dispatchPushStream(quic::StreamId /* pushStreamId */,
+                          hq::PushId /* pushId */,
+                          size_t /* toConsume */) override;
 
   // Incoming server push streams. Since the incoming push streams
   // can be created before transport stream
