@@ -21,7 +21,6 @@
 #include <folly/io/async/HHWheelTimer.h>
 #include <proxygen/lib/http/HTTPConstants.h>
 #include <proxygen/lib/http/HTTPHeaderSize.h>
-#include <proxygen/lib/http/codec/ControlMessageRateLimitFilter.h>
 #include <proxygen/lib/http/codec/FlowControlFilter.h>
 #include <proxygen/lib/http/codec/HTTPCodec.h>
 #include <proxygen/lib/http/codec/HTTPCodecFilter.h>
@@ -160,6 +159,10 @@ class HTTPSession
     return HTTPSessionBase::getSetupTransportInfo();
   }
 
+  [[nodiscard]] std::chrono::seconds getLatestIdleTime() const override {
+    return HTTPSessionBase::getLatestIdleTime();
+  }
+
   bool getCurrentTransportInfo(wangle::TransportInfo* tinfo) override;
 
   void getFlowControlInfo(HTTPTransaction::FlowControlInfo* info) override;
@@ -244,6 +247,8 @@ class HTTPSession
   bool getCurrentTransportInfoWithoutUpdate(
       wangle::TransportInfo* tinfo) const override;
 
+  void setHeaderIndexingStrategy(const HeaderIndexingStrategy* strat) override;
+
   void setHeaderCodecStats(HeaderCodec::Stats* stats) override {
     codec_->setHeaderCodecStats(stats);
   }
@@ -270,15 +275,6 @@ class HTTPSession
   void setSecondAuthManager(
       std::unique_ptr<SecondaryAuthManagerBase> secondAuthManager);
 
-  void setControlMessageRateLimitParams(
-      uint32_t maxControlMsgsPerInterval = kDefaultMaxControlMsgsPerInterval,
-      uint32_t maxDirectErrorHandlingPerInterval =
-          kDefaultMaxDirectErrorHandlingPerInterval,
-      std::chrono::milliseconds controlMsgIntervalDuration =
-          kDefaultControlMsgDuration,
-      std::chrono::milliseconds directErrorHandlingIntervalDuration =
-          kDefaultDirectErrorHandlingDuration);
-
   /**
    * Get the SecondaryAuthManager attached to this session.
    */
@@ -771,8 +767,6 @@ class HTTPSession
 
   WheelTimerInstance wheelTimer_;
 
-  ControlMessageRateLimitFilter* controlMessageRateLimitFilter_{nullptr};
-
   /**
    * Number of writes submitted to the transport for which we haven't yet
    * received completion or failure callbacks.
@@ -920,7 +914,7 @@ class HTTPSession
 
   void scheduleResetDirectErrorHandling();
 
-  size_t sendPing(uint64_t data);
+  size_t sendPing(uint64_t data) override;
 
   // private members
 
@@ -1126,6 +1120,13 @@ class HTTPSession
     }
     ~ObserverAccessor() override = default;
 
+    size_t sendPing(uint64_t data) override {
+      if (sessionBasePtr_) {
+        return sessionBasePtr_->sendPing(data);
+      }
+      return 0;
+    }
+
    private:
     HTTPSessionBase* sessionBasePtr_{nullptr};
   };
