@@ -12,7 +12,9 @@
 #include <folly/io/IOBuf.h>
 #include <folly/io/async/AsyncUDPSocket.h>
 #include <folly/io/async/SSLContext.h>
+#include <proxygen/lib/http/codec/ControlMessageRateLimitFilter.h>
 #include <proxygen/lib/http/codec/HTTPCodecFilter.h>
+#include <proxygen/lib/http/codec/RateLimitFilter.h>
 #include <proxygen/lib/http/observer/HTTPSessionObserverContainer.h>
 #include <proxygen/lib/http/observer/HTTPSessionObserverInterface.h>
 #include <proxygen/lib/http/session/HTTPSessionActivityTracker.h>
@@ -171,10 +173,17 @@ class HTTPSessionBase : public wangle::ManagedConnection {
     infoCallback_ = callback;
   }
 
+  void setRateLimitParams(RateLimitFilter::Type type,
+                          uint32_t maxEventsPerInterval,
+                          std::chrono::milliseconds intervalDuration);
+
   InfoCallback* getInfoCallback() const {
     return infoCallback_;
   }
 
+  virtual void setHeaderIndexingStrategy(
+      const HeaderIndexingStrategy* strat) = 0;
+
   virtual void setSessionStats(HTTPSessionStats* stats);
 
   virtual HTTPTransaction::Transport::Type getType() const noexcept = 0;
@@ -239,6 +248,20 @@ class HTTPSessionBase : public wangle::ManagedConnection {
     codec_.add<Filter>(std::forward<Args>(args)...);
   }
 
+  void addRateLimitFilter(RateLimitFilter::Type type) {
+    CHECK_LT(folly::to_underlying(type),
+             folly::to_underlying(RateLimitFilter::Type::MAX))
+        << "Received a rate limit type that exceeded the specified maximum";
+    if (!rateLimitFilters_[folly::to_underlying(type)]) {
+      auto filter = RateLimitFilter::createRateLimitFilter(
+          type, &getEventBase()->timer(), sessionStats_);
+      CHECK(filter) << "Unable to construct a rate limit filter of type "
+                    << RateLimitFilter::toStr(type);
+      rateLimitFilters_[folly::to_underlying(type)] = filter.get();
+      codec_.addFilters(std::move(filter));
+    }
+  }
+
   virtual CodecProtocol getCodecProtocol() const {
     return codec_->getProtocol();
   }
@@ -325,6 +348,8 @@ class HTTPSessionBase : public wangle::ManagedConnection {
    */
   virtual size_t sendPing() = 0;
 
+  virtual size_t sendPing(uint64_t data) = 0;
+
   /**
    * Sends a priority message on this session.  If the underlying protocol
    * doesn't support priority, this is a no-op.  A new stream identifier will
@@ -355,17 +380,17 @@ class HTTPSessionBase : public wangle::ManagedConnection {
     return transactionSeqNo_;
   }
 
-  std::chrono::seconds getLatestIdleTime() const {
+  [[nodiscard]] bool getStreamLimitExceeded() const {
+    return streamLimitExceeded_;
+  }
+
+  [[nodiscard]] std::chrono::seconds getLatestIdleTime() const /*override*/ {
     DCHECK_GT(transactionSeqNo_, 0u)
         << "No idle time for the first transaction";
     DCHECK(latestActive_ > TimePoint::min());
     return latestIdleDuration_;
   }
 
-  void setPrioritySampled(bool sampled) {
-    prioritySample_ = sampled;
-  }
-
   // public HTTPTransaction::Transport overrides
   virtual const folly::SocketAddress& getLocalAddress() const noexcept {
     return localAddr_;
@@ -392,10 +417,6 @@ class HTTPSessionBase : public wangle::ManagedConnection {
     return transportInfo_;
   }
 
-  virtual void onNetworkSwitch(
-      std::unique_ptr<folly::AsyncUDPSocket>) noexcept {
-  }
-
   /**
    * If the connection is closed by remote end
    */
@@ -452,13 +473,14 @@ class HTTPSessionBase : public wangle::ManagedConnection {
 
   virtual bool isDetachable(bool checkSocket) const = 0;
 
-  virtual void attachThreadLocals(folly::EventBase* eventBase,
-                                  folly::SSLContextPtr sslContext,
-                                  const WheelTimerInstance& wheelTimer,
-                                  HTTPSessionStats* stats,
-                                  FilterIteratorFn fn,
-                                  HeaderCodec::Stats* headerCodecStats,
-                                  HTTPSessionController* controller) = 0;
+  virtual void attachThreadLocals(
+      folly::EventBase* eventBase,
+      std::shared_ptr<const folly::SSLContext> sslContext,
+      const WheelTimerInstance& wheelTimer,
+      HTTPSessionStats* stats,
+      FilterIteratorFn fn,
+      HeaderCodec::Stats* headerCodecStats,
+      HTTPSessionController* controller) = 0;
 
   virtual void detachThreadLocals(bool detachSSLContext = false) = 0;
 
@@ -540,6 +562,23 @@ class HTTPSessionBase : public wangle::ManagedConnection {
     return false;
   }
 
+  /**
+   * Adds an observer.
+   *
+   * If the observer is already added, this is a no-op.
+   *
+   * @param observer     <shared_ptr> Observer to add
+   * @return             Whether the observer was added (fails if no list).
+   */
+  bool addObserver(
+      std::shared_ptr<HTTPSessionObserverContainer::Observer> observer) {
+    if (auto list = getHTTPSessionObserverContainer()) {
+      list->addObserver(std::move(observer));
+      return true;
+    }
+    return false;
+  }
+
   /**
    * Removes an observer.
    *
@@ -556,23 +595,17 @@ class HTTPSessionBase : public wangle::ManagedConnection {
   }
 
   /**
-   * Returns the HTTPSessionObserverContainer or nullptr if not available.
+   * Removes an observer.
    *
-   * HTTPSession implementations that support observers should override this
-   * function and return the session observer container that they hold in
-   * memory.
-
-   *
-   * We have a default implementation to ensure that there is no risk of a
-   * pure-virtual function being called during constructon or destruction of
-   * the session. If this was to occur the derived class which implements this
-   * function may be unavailable leading to undefined behavior. While this is
-   * true for any pure-virtual function, the potential for this issue is
-   * greater for observers.
+   * @param observer     Observer to remove.
+   * @return             Whether the observer was found and removed.
    */
-  [[nodiscard]] virtual HTTPSessionObserverContainer*
-  getHTTPSessionObserverContainer() const {
-    return nullptr;
+  bool removeObserver(
+      std::shared_ptr<HTTPSessionObserverContainer::Observer> observer) {
+    if (auto list = getHTTPSessionObserverContainer()) {
+      return list->removeObserver(std::move(observer));
+    }
+    return false;
   }
 
  protected:
@@ -586,6 +619,10 @@ class HTTPSessionBase : public wangle::ManagedConnection {
     return pendingWriteSize_ + pendingWriteSizeDelta_ > 0;
   }
 
+  uint64_t getPendingWriteSize() const {
+    return pendingWriteSize_;
+  }
+
   /**
    * Install a direct response handler for the transaction based on the
    * error.
@@ -617,8 +654,8 @@ class HTTPSessionBase : public wangle::ManagedConnection {
     ++transactionSeqNo_;
   }
 
-  bool isPrioritySampled() const {
-    return prioritySample_;
+  void setStreamLimitExceeded(bool streamLimitExceeded) {
+    streamLimitExceeded_ = streamLimitExceeded;
   }
 
   void onNewOutgoingStream(uint32_t outgoingStreams) {
@@ -665,6 +702,25 @@ class HTTPSessionBase : public wangle::ManagedConnection {
    */
   void informSessionControllerTransportReady();
 
+  /**
+   * Returns the HTTPSessionObserverContainer or nullptr if not available.
+   *
+   * HTTPSession implementations that support observers should override this
+   * function and return the session observer container that they hold in
+   * memory.
+   *
+   * We have a default implementation to ensure that there is no risk of a
+   * pure-virtual function being called during constructon or destruction of
+   * the session. If this was to occur the derived class which implements this
+   * function may be unavailable leading to undefined behavior. While this is
+   * true for any pure-virtual function, the potential for this issue is
+   * greater for observers.
+   */
+  [[nodiscard]] virtual HTTPSessionObserverContainer*
+  getHTTPSessionObserverContainer() const {
+    return nullptr;
+  }
+
   HTTPSessionStats* sessionStats_{nullptr};
 
   InfoCallback* infoCallback_{nullptr}; // maybe can move to protected
@@ -718,10 +774,13 @@ class HTTPSessionBase : public wangle::ManagedConnection {
 
   std::unique_ptr<HTTPSessionActivityTracker> httpSessionActivityTracker_;
 
+  std::array<RateLimitFilter*, folly::to_underlying(RateLimitFilter::Type::MAX)>
+      rateLimitFilters_{};
+
  private:
-  // Underlying controller_ is marked as private so that callers must utilize
-  // getController/setController protected methods.  This ensures we have a
-  // single path to update controller_
+  // Underlying controller_ is marked as private so that callers must
+  // utilize getController/setController protected methods.  This ensures we
+  // have a single path to update controller_
   HTTPSessionController* controller_{nullptr};
 
   // private ManagedConnection methods
@@ -790,13 +849,17 @@ class HTTPSessionBase : public wangle::ManagedConnection {
    */
   uint32_t pendingReadSize_{0};
 
-  bool prioritySample_ : 1;
   bool h2PrioritiesEnabled_ : 1;
 
   /**
    * Indicates whether Ex Headers is supported in HTTPSession
    */
   bool exHeadersEnabled_ : 1;
+
+  /**
+   * Indicates whether quic stream ID limit is exceeded.
+   */
+  bool streamLimitExceeded_{false};
 };
 
 } // namespace proxygen
