@@ -44,6 +44,7 @@ struct TestParams {
   std::size_t numBytesOnPushStream{kUnlimited};
   bool expectOnTransportReady{true};
   bool datagrams_{false};
+  bool webTransport_{false};
   bool checkUniridStreamCallbacks{true};
 };
 
@@ -122,6 +123,12 @@ class HQSessionTest
     if (GetParam().datagrams_) {
       egressSettings_.setSetting(proxygen::SettingsId::_HQ_DATAGRAM, 1);
     }
+    if (GetParam().webTransport_) {
+      egressSettings_.setSetting(proxygen::SettingsId::_HQ_DATAGRAM, 1);
+      egressSettings_.setSetting(proxygen::SettingsId::ENABLE_CONNECT_PROTOCOL,
+                                 1);
+      egressSettings_.setSetting(proxygen::SettingsId::ENABLE_WEBTRANSPORT, 1);
+    }
 
     egressControlCodec_ = std::make_unique<proxygen::hq::HQControlCodec>(
         nextUnidirectionalStreamId_,
@@ -265,6 +272,8 @@ class HQSessionTest
           }
         }
           return;
+        case proxygen::hq::UnidirectionalStreamType::WEBTRANSPORT:
+          return;
         default:
           CHECK(false) << "Unknown stream preface=" << preface->first;
       }
@@ -312,6 +321,9 @@ class HQSessionTest
 
   struct MockControllerContainer {
     explicit MockControllerContainer(TestParams params) {
+      EXPECT_CALL(mockController, getHeaderIndexingStrategy())
+          .WillRepeatedly(testing::Return(
+              proxygen::HeaderIndexingStrategy::getDefaultInstance()));
       testing::InSequence s;
       EXPECT_CALL(mockController, attachSession(testing::_));
       if (params.expectOnTransportReady) {
