@@ -30,6 +30,14 @@ class MockSessionObserver
               requestStarted,
               (HTTPSessionObserverAccessor*, const RequestStartedEvent&),
               (noexcept));
+  MOCK_METHOD(void,
+              preWrite,
+              (HTTPSessionObserverAccessor*, const PreWriteEvent&),
+              (noexcept));
+  MOCK_METHOD(void,
+              pingReply,
+              (HTTPSessionObserverAccessor*, const PingReplyEvent&),
+              (noexcept));
   ~MockSessionObserver() override = default;
 };
 } // namespace proxygen
