@@ -24,6 +24,8 @@ class ConnectionStats {
 
   virtual void recordConnectionOpen() = 0;
 
+  virtual void recordTcpConnectionOpen() = 0;
+
   virtual void recordConnectionClose() = 0;
 
   virtual void recordRequest() = 0;
@@ -48,10 +50,12 @@ class ConnectionStats {
  */
 class TLConnectionStats : public ConnectionStats {
  public:
-  explicit TLConnectionStats(const std::string& prefix);
+  explicit TLConnectionStats(const std::string& prefix, uint8_t verbosity);
 
   void recordConnectionOpen() override;
 
+  void recordTcpConnectionOpen() override;
+
   void recordConnectionClose() override;
 
   void recordRequest() override;
@@ -70,17 +74,20 @@ class TLConnectionStats : public ConnectionStats {
   void addIngressBodyBytes(size_t bytes) override;
 
  private:
-  BaseStats::TLTimeseriesMinuteAndAllTime req_;
-  BaseStats::TLTimeseriesMinuteAndAllTime resp_;
-  BaseStats::TLTimeseriesMinuteAndAllTime egressBytes_;
-  BaseStats::TLTimeseriesMinuteAndAllTime ingressBytes_;
-  BaseStats::TLTimeseriesMinuteAndAllTime egressBodyBytes_;
-  BaseStats::TLTimeseriesMinuteAndAllTime ingressBodyBytes_;
-  TLResponseCodeStats responseCodes_;
-  BaseStats::TLHistogram totalDuration_;
+  std::optional<BaseStats::TLTimeseriesMinuteAndAllTime> req_;
+  std::optional<BaseStats::TLTimeseriesMinuteAndAllTime> resp_;
+  std::optional<BaseStats::TLTimeseriesMinuteAndAllTime> egressBytes_;
+  std::optional<BaseStats::TLTimeseriesMinuteAndAllTime> ingressBytes_;
+  std::optional<BaseStats::TLTimeseriesMinuteAndAllTime> egressBodyBytes_;
+  std::optional<BaseStats::TLTimeseriesMinuteAndAllTime> ingressBodyBytes_;
+  std::optional<TLResponseCodeStats> responseCodes_;
+  std::optional<BaseStats::TLHistogram> totalDuration_;
 
-  BaseStats::TLCounter currConns_;
-  BaseStats::TLTimeseries newConns_;
+  std::optional<BaseStats::TLCounter> currConns_;
+  std::optional<BaseStats::TLTimeseries> newConns_;
+
+  std::optional<BaseStats::TLCounter> currTcpConns_;
+  std::optional<BaseStats::TLTimeseries> newTcpConns_;
 };
 
 } // namespace proxygen
